use git2::Repository;
use semver::Version;
use git2::string_array::StringArray;
use regex::Regex;

pub struct GitVersion<'a> {
  pub dir: &'a str,
  pub prefix: &'a str,
  version: String,
}

impl GitVersion<'_> {
  pub fn new<'a>(dir: &'a str, prefix: &'a str) -> GitVersion<'a> {
    GitVersion {
      dir,
      prefix,
      version: String::from("0.0.0"),
    }
  }

  pub fn current_version(&mut self) -> String {
    let repo = Repository::open(self.dir)
        .map_err(|e| format!("Failed to open: {}", e))
        .unwrap();

    let all_tags = self.git_tag(&repo).unwrap();

    // filter by a specific regexp
    let pattern = format!("{}{}{}", '^', self.prefix, "\\d+\\.\\d+\\.\\d+$");
    let tag_regex = Regex::new(pattern.as_ref()).unwrap();

    // sort git versions
    let mut version_tags: Vec<&str> = all_tags
        .iter()
        .filter(|tag| tag_regex.is_match(tag.unwrap()))
        .map(|tag| tag.unwrap())
        .collect();
    version_tags.sort_by(|a, b| natord::compare(&a, &b));

    // find the greatest version
    let tag_list_length = version_tags.len();

    if tag_list_length != 0 {
      self.version = String::from(version_tags[tag_list_length - 1]);
    }

    // version
    self.version.to_string()
  }

  pub fn next_version(&mut self, typ: &str) -> String {
    let mut version = Version::parse(&self.current_version()).unwrap();

    match typ {
      "major" => version.increment_major(),
      "minor" => version.increment_minor(),
      "patch" => version.increment_patch(),
      "none" => (),
      _ => unreachable!()
    }

    self.version = version.to_string();

    format!("{}{}", self.prefix, self.version)
  }

  fn git_tag(&mut self, repo: &git2::Repository) -> Result<StringArray, git2::Error> {
    repo.tag_names(None)
  }
}
